//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "video.h"
#include "sound.h"
#include "player.h"
#include "fontsHelper.h"
#include "road.h"
#include "board.h"
#include "toolkit.h"

/*************** Sprites includes *************/
#include ".\gen_src\weapon.h"
#include ".\gen_src\arrow.h"
#include ".\gen_src\bonus.h"
#include ".\gen_src\progress.h"

/*************** Compress Sprites includes ****/
#include ".\gen_src\board_0.h"
#include ".\gen_src\board_1.h"
#include ".\gen_src\damages.h"

/*************** Global variables *************/
u8 gArmorColors[NB_ARMOR_COLORS];
                                                
/*************** Local defines ****************/
#define ARROW_X					20
#define ARROW_Y					BOARD_CY-7
#define ARROW_CX				G_ARROW_0_W
#define ARROW_CY				G_ARROW_0_H

#define BOARD_WEAPON_Y			4
#define BOARD_WEAPON_X			3
#define BOARD_WEAPON_OFFS_Y		10
#define BOARD_WEAPON_CX			G_WEAPON_0_W
#define BOARD_WEAPON_CY			G_WEAPON_0_H
#define BOARD_WEAPON_SIZE		(BOARD_WEAPON_CX*BOARD_WEAPON_CY*2)

#define BOARD_MINE_X			8
#define BOARD_MINE_Y			14
#define BOARD_MISSILE_X			8
#define BOARD_MISSILE_Y			24

#define BONUS_X					28
#define BONUS_Y					16
#define BONUS_CX				GBONUS_BONUS_0_W
#define BONUS_CY				GBONUS_BONUS_0_H

#define LINE_X					BONUS_X
#define LINE_Y					(BONUS_Y + BONUS_CY + 1)
#define LINE_CX					1
#define LINE_CY					1
#define PROGRESS_SIZE			6

#define DAMAGES_X				43
#define DAMAGES_Y				9
#define DAMAGES_CX				G_DAMAGES_W
#define DAMAGES_CY				G_DAMAGES_H

#define FILL_DAMAGES_X			DAMAGES_X + 1
#define FILL_DAMAGES_Y			DAMAGES_Y + 2
#define FILL_DAMAGES_CX			10
#define FILL_DAMAGES_CY			15

#define LIFES_X					72
#define LIFES_Y					7

#define SCORE_X					66
#define SCORE_Y					22

#define MAX_WEAPON_CHARGE		9
#define NB_WEAPONS				3

#define MAX_HEAT				29
#define MIN_HEAT				0
#define SAFE_HEAT				7

/*****************************************************/
/*													 */
/*	Update life info								 */
/*												     */
/*****************************************************/
void UpdateInfoLife()
{
	PrintInt(LIFES_X, LIFES_Y, gRoad._player._life, 2);
}

/*****************************************************/
/*													 */
/*	Update armor color								 */
/*												     */
/*****************************************************/
void UpdateInfoArmor()
{
	u8 y;
	u8 armor = GET_PLAYER_CAR._armor / ((MAX_ARMOR + 5) / NB_ARMOR_COLORS);

	/** Draw armor color */
	cpct_drawSolidBox(GetBackScreenPtr(FILL_DAMAGES_X, FILL_DAMAGES_Y), gArmorColors[armor], FILL_DAMAGES_CX, FILL_DAMAGES_CY);

	/** Draw car part */
	DrawSpriteType(g_damages, GetBackScreenPtr(DAMAGES_X, DAMAGES_Y), DAMAGES_CX, DAMAGES_CY, SPRITE_MASKED);

	/** Copy to front */
	for (y = DAMAGES_Y; y < DAMAGES_CY + DAMAGES_Y; y++)
	{
		u8* to = GetScreenPtr(DAMAGES_X, y);
		u8* from = GetBackScreenPtr(DAMAGES_X, y);
		cpct_memcpy(to, from, DAMAGES_CX);
	}
}

/*****************************************************/
/*													 */
/*	Init board										 */
/*												     */
/*****************************************************/
void InitBoard()
{
	u8* tempSpriteBuffer0 = WinCpcTelera_GetMemPtr((u16)SCREEN_BUFF);
	u8* tempSpriteBuffer1 = WinCpcTelera_GetMemPtr((u16)(SCREEN_BUFF + BOARD_CX * BOARD_CY));

	/** Extract in backBuffer memory */
	Decompress(G_board_0, tempSpriteBuffer0);
	Decompress(G_board_1, tempSpriteBuffer1);

	/* Fill road color */
	cpct_memset(CPCT_VMEM_START, gRoadColor, SCREEN_SIZE);

	/** Draw to videoBuffer board */
	cpct_drawSprite(tempSpriteBuffer0, cpctm_screenPtr(CPCT_VMEM_START, 0, 0), BOARD_CX, BOARD_CY);
	cpct_drawSprite(tempSpriteBuffer1, cpctm_screenPtr(CPCT_VMEM_START, SCREEN_CX_BYTE / 2, 0), BOARD_CX, BOARD_CY);

	/** Draw arrow */
	DrawSpriteBothBuffers(g_arrow_0, ARROW_X, ARROW_Y, ARROW_CX, ARROW_CY, SPRITE_OPAQUE);
}

/*****************************************************/
/*													 */
/*	Update board weapon selected					 */
/*												     */
/*****************************************************/
void UpdateInfoWeapon()
{
	static u8* const sWeaponInfo[] = { g_weapon_0, g_weapon_1, g_weapon_2, g_weapon_3, g_weapon_4, g_weapon_5 };

	u8 i;
	for (i = 0; i < NB_WEAPONS; i++)
		DrawSpriteBothBuffers(sWeaponInfo[(i == GET_PLAYER._weaponSelect) ? i + 3: i], BOARD_WEAPON_X, BOARD_WEAPON_Y + i*BOARD_WEAPON_OFFS_Y, BOARD_WEAPON_CX, BOARD_WEAPON_CY, SPRITE_MASKED);
}

/*****************************************************/
/*													 */
/*	Update dashboard ammunition						 */
/*												     */
/*****************************************************/
void UpdateInfoAmmo()
{
	PrintInt(BOARD_MINE_X, BOARD_MINE_Y, GET_PLAYER._mines, 2);
	PrintInt(BOARD_MISSILE_X, BOARD_MISSILE_Y, GET_PLAYER._missiles, 2);
}

/*****************************************************/
/*													 */
/*	Update bonus									 */
/*												     */
/*****************************************************/
void UpdateInfoBonus()
{
	DrawSpriteBothBuffers(gBonus_tileset[GET_PLAYER._bonus._id], BONUS_X, BONUS_Y, BONUS_CX, BONUS_CY, SPRITE_OPAQUE);
	UpdateCounterBonus();
}

/*****************************************************/
/*													 */
/*	Update bonus									 */
/*												     */
/*****************************************************/
void UpdateCounterBonus()
{
	u8 counter = GET_PLAYER._bonus._counter / (BONUS_COUNTER / PROGRESS_SIZE);
	u8 i;
	for (i = 0; i < PROGRESS_SIZE; i++)
		DrawSpriteBothBuffers((i + 1) > counter ? g_progress_0 : g_progress_1, LINE_X + i, LINE_Y, LINE_CX, LINE_CY, SPRITE_OPAQUE);
}

/*****************************************************/
/*													 */
/*	Update dashboard score							 */
/*												     */
/*****************************************************/
void UpdateInfoScore()
{
	PrintInt(SCORE_X, SCORE_Y, GET_PLAYER._money, 4);
}

/*****************************************************/
/*													 */
/*	Update dashboard gun heat						 */
/*												     */
/*****************************************************/
void UpdateHeat()
{
	u8 heat = GET_PLAYER._heat;

	if (GET_PLAYER_CAR._status & STATUS_SHOOT && GET_PLAYER_CAR._waitTime == FALSE)
	{
		if (GET_PLAYER._heat != MAX_HEAT)
			GET_PLAYER._heat++;
		else
		{
			GET_PLAYER_CAR._waitTime = TRUE;
			GET_PLAYER_OBJ._counter = 1;
		}
	}
	else
	if (GET_PLAYER._heat != MIN_HEAT)
	{
		if (--GET_PLAYER._heat == SAFE_HEAT)
			GET_PLAYER_CAR._waitTime = FALSE;
	}

	if (heat != GET_PLAYER._heat)
		DrawSpriteBothBuffers(GET_PLAYER_CAR._waitTime ? g_arrow_1 : g_arrow_0, ARROW_X, ARROW_Y - GET_PLAYER._heat, ARROW_CX, ARROW_CY, SPRITE_OPAQUE);
}