//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>
#include <limits.h>

/*************** Game includes ****************/
#include "structures.h"
#include "fontsHelper.h"
#include "toolkit.h"
#include "sound.h"
#include "video.h"
#include "cars.h"
#include "player.h"
#include "objects.h"
#include "mines.h"
#include "missiles.h"
#include "explosions.h"
#include "gun.h"
#include "road.h"
#include "menus.h"

/*************** Local defines ****************/
#define UPDATE_TIME			12	// interrupts

/*************** Static variables *************/
static u8 sTimer;
static u8 sTimerUpdate;
static u8 sInterrupt;

/*************** Extern variables *************/
extern const u8 gGamePalette[];
extern const u8 gBoardPalette[];
extern u8 gMode;
extern SRoad gRoad;

/*************** Interrupt handler Timer ******/
void IncrInterrupt()
{
	sTimer++;

	if (sInterrupt == 4)
		PlaySound();
		
	if (++sInterrupt == 6)
	{
		sInterrupt = 0;
		cpct_scanKeyboard_if();
	}
}

/*************** Interrupt handler Game ******/
static void sInterruptGameHandlerTimer(void)
{
	if (gMode & GAME)
	{
		/** Board or Game palette */
		SetFullPalette(sInterrupt == 1 ? gBoardPalette : gGamePalette);
	}
	else
		SetAllPaletteBlack();

	IncrInterrupt();
}

/*************** Interrupt handler End ******/
static void sInterruptEndGameHandlerTimer(void)
{
	cpct_setVideoMode(sInterrupt == 1 ? 1 : 0);
	
	cpct_setPALColour(1, sInterrupt == 1 ? 0x54 : 0x40);
	cpct_setPALColour(2, sInterrupt == 1 ? 0x4B : 0x4B);

	IncrInterrupt();
}

/*****************************************************/
/*													 */
/*	Set game interruption							 */
/*												     */
/*****************************************************/
void SetGameInterrupt()
{
	InitSFX();	
	SetInterruptHandler(sInterruptGameHandlerTimer);
}

/*****************************************************/
/*													 */
/*	Initialize game									 */
/*												     */
/*****************************************************/
void InitGame()
{
	gRoad._curRound = 0;
	gRoad._curLevel = 0;

	SetDisplay(TRUE);
	cpct_setBorder(HW_BLACK);

	DeleteCars();
	InitPlayerNewGame();

	INIT_MINE;
	INIT_OBJECTS;
	INIT_EXPLOSIONS;

	sTimer = 0;
	sTimerUpdate = 0;
}

/*****************************************************/
/*													 */
/*	End game										 */
/*												     */
/*****************************************************/
void EndGame()
{
	if (!(gMode & WIN))
	{
		DrawEndGame();

		SetInterruptHandler(sInterruptEndGameHandlerTimer);
		SetFullPalette(gGamePalette);
		cpct_setPALColour(7, 0x53);

		GET_PLAYER_OBJ._spriteInfo._y = PLAYER_END_Y;
		GET_PLAYER_OBJ._spriteInfo._x = PLAYER_ENTER_X;

		GET_PLAYER_CAR._status = STATUS_ENTER;
	}
}

/*****************************************************/
/*													 */
/*	Test if timer activated							 */
/*												     */
/*****************************************************/
BOOL TestTimer(u8* curTime, u8 timeToTest)
{
	u8 elapse;
	i8 computed = sTimer - *curTime;

	if (computed < 0)
		elapse = UCHAR_MAX + computed;
	else
		elapse = computed;

	if (elapse > timeToTest)
	{
		*curTime = sTimer;
		return TRUE;
	}
	return FALSE;
}

/*****************************************************/
/*													 */
/*	Game entry										 */
/*												     */
/*****************************************************/
void Game()
{
	if (gMode & GAME)
	{
		UpdatePlayerInput();

		if (gMode & GAME)
		{
			if (TestTimer(&sTimerUpdate, UPDATE_TIME))
			{
				UpdatePlayer();
				UpdateCars();
			}

			ClearPlayer();
			ClearCars();

			CLEAR_EXPLOSIONS;
			CLEAR_MINES;
			CLEAR_MISSILES;
			CLEAR_ROAD_OBJECTS;

			DrawImpacts();
			DrawRoadBorder();
			DrawObjects();
			DrawExplosions();
			DrawMines();
			DrawMissiles();
			DrawPlayer();
			DrawCars();
		}
	}
	else
	{
		if (cpct_isAnyKeyPressed())
			InitMenu(TRUE);
		else
		{
			UpdatePlayerInput();

			ClearPlayer();
			DrawPlayer();
			DrawRoadBorder();
		}
	}

	if (gMode & (WIN | GAME))
		FlipBuffer();
}