//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "video.h"
#include "sound.h"
#include "road.h"
#include "player.h"
#include "cars.h"
#include "gun.h"
#include "explosions.h"
#include "toolkit.h"

/*************** Sprites includes *************/
#include ".\gen_src\impact.h"
#include ".\gen_src\shoot.h"
#include ".\gen_src\shoot_double.h"

/*************** static variables *************/
static SImpacts sImpacts;

/*************** extern variables *************/
extern const SCarInfo gCarInfos[];

/*************** Local defines ***************/
#define IMPACT_CX			G_IMPACT_0_W
#define IMPACT_CY			G_IMPACT_0_H

#define SHOOT_CX			G_SHOOT_0_W
#define SHOOT_CY			G_SHOOT_0_H
#define SHOOT_D_CY			G_SHOOT_DOUBLE_0_H

#define IMPACT_TIME			3
#define SHOOT_RELOAD_TIME	10
#define SHOOT_EXPLO_TIME	30

void DrawShoot(SCar *carShooter);

/*****************************************************/
/*													 */
/*	Init impacts									 */
/*												     */
/*****************************************************/
void InitImpacts()
{
	sImpacts._nbImpacts = 0;
}

/*****************************************************/
/*													 */
/*	Get shoot orientation							 */
/*												     */
/*****************************************************/
u8 GetShootDirection(u8 gun)
{
	if (gun & GUN_RIGHT)
		return RIGHT;
	else if (gun & GUN_LEFT)
		return LEFT;
	else if (gun & GUN_UP)
		return UP;

	return DOWN;
}

/*****************************************************/
/*													 */
/*	Test if hit car									 */
/*												     */
/*****************************************************/
BOOL IsCarHit(SCar* car, u8 weapon, u8* shootX, u8* shootY)
{
	const SSpriteInfo* sprite = &car->_roadObj._spriteInfo;
	u8 posX = sprite->_x;
	if (posX < SCREEN_CX_BYTE)
	{
		u8 direction = GetShootDirection(weapon);
		u8 posY = sprite->_y;

		u8 shotX = *shootX;
		u8 shotY = *shootY;

		/** Horizontal shoot */
		if (direction == RIGHT || direction == LEFT)
		{
			if (shotY > posY && shotY < posY + sprite->_cy)
			{
				/** Shoot to RIGHT test */
				if (direction == RIGHT && shotX < posX && posX > 0)
				{
					*shootX = posX - IMPACT_CX / 2;
					return TRUE;
				}

				/** Shoot to LEFT test */
				if (direction == LEFT && shotX > posX && posX < SCREEN_CX_BYTE)
				{
					*shootX = posX + sprite->_cx - IMPACT_CX / 2;
					return TRUE;
				}
			}
		}
		/** Vertical shoot */
		else
		{
			if (shotX > posX && shotX < posX + sprite->_cx)
			{
				/** Shoot to UP test */
				if (direction == UP && shotY > (u8)posY)
				{
					*shootY = (u8)posY + sprite->_cy - IMPACT_CY / 2;
					return TRUE;
				}

				/** Shoot to DOWN test */
				if (direction == DOWN && shotY < (u8)posY)
				{
					*shootY = (u8)posY;
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

/*****************************************************/
/*													 */
/*	Update shoot activity							 */
/*												     */
/*****************************************************/
void UpdateShoot(SCar* car)
{
	i8* counter = &car->_roadObj._counter;

	if (car->_status != STATUS_SHOOT)
	{
		*counter = GUN_START_COUNTER;
		car->_status = STATUS_SHOOT;
	}
	else
	{
		if (*counter <= GUN_END_COUNTER)
			*counter = GUN_START_COUNTER;
	}

	PlaySFX(FX_SHOOT, FX_SHOOT_CHANNEL);
}

/*****************************************************/
/*													 */
/*	Update shoot counter							 */
/*												     */
/*****************************************************/
void UpdateShootCounter(SCar *car)
{
	if (++car->_roadObj._data == SHOOT_RELOAD_TIME)
	{
		if (car->_nbShoot > 0)
		{
			car->_nbShoot--;
			car->_roadObj._data = 0;
			UpdateShoot(car);
		}
	}
}

/*****************************************************/
/*													 */
/*	Add impact										 */
/*												     */
/*****************************************************/
void AddImpact(u8 shootX, u8 shootY)
{
	if (sImpacts._nbImpacts < NB_IMPACTS_MAX)
	{
		SImpact* newImpact = &sImpacts._impact[sImpacts._nbImpacts++];
		newImpact->_time = IMPACT_TIME;

		newImpact->_x = shootX;
		newImpact->_y = shootY - IMPACT_CY/2;
	}

	PlaySFX(FX_IMPACT, FX_IMPACT_CHANNEL);
}

/*****************************************************/
/*													 */
/*	Delete Impact									 */
/*												     */
/*****************************************************/
void DeleteImpact(SImpact* impactToDel)
{
	if (impactToDel->_time < TIME_TO_DEL)
	{
		const SImpact* lastImpact = &sImpacts._impact[--sImpacts._nbImpacts];
		if (impactToDel != lastImpact)
			cpct_memcpy(impactToDel, lastImpact, sizeof(SImpact));
	}
}

/*****************************************************/
/*													 */
/*	Delete impacts									 */
/*												     */
/*****************************************************/
void DeleteImpacts()
{
	SImpact* impact = sImpacts._impact;

	u8 i;
	for (i = 0; i < sImpacts._nbImpacts; i++)
	{
		/** Delete impact */
		DeleteImpact(impact);
		impact++;
	}
}

/*****************************************************/
/*													 */
/*	Draw Impacts									 */
/*												     */
/*****************************************************/
void DrawImpacts()
{
	SImpact* impact = sImpacts._impact;

	u8 i;
	for (i = 0; i < sImpacts._nbImpacts; i++)
	{
 		i16 time = --impact->_time;

		/** Draw impact */
		if (time > TIME_TO_DEL)
		{
			const u8* spriteImpact = g_impact_0;
			if (time % 2 == 0)
				spriteImpact = g_impact_1;

			DrawSpriteType(spriteImpact, GetScreenPtr(impact->_x, impact->_y), IMPACT_CX, IMPACT_CY, SPRITE_MASKED);
		}
		else
			/** Clear impact */
			ClearScreenZone(impact->_x, impact->_y, IMPACT_CX, IMPACT_CY, BACK_BUFFER);
			
		impact++;
	}

	DeleteImpacts();
}

/*****************************************************/
/*													 */
/*	Test if hit something							 */
/*												     */
/*****************************************************/
BOOL TestHit(SCar* carShooter, u8 shootX, u8 shootY)
{
	u8 nbCar = gRoad._cars._nbCars;
	if (nbCar > 0)
	{
		if (carShooter != GET_PLAYER_CAR_PTR)
		{
			if (IsCarHit(GET_PLAYER_CAR_PTR, carShooter->_gun, &shootX, &shootY))
			{
				AddImpact(shootX, shootY);
				DamageCar(GET_PLAYER_CAR_PTR, SHOOT_DAMAGES);
				return TRUE;
			}
		}
		else
		{
			SCar* itCar = gRoad._cars._car;

			u8 i;
			for (i = 0; i < nbCar; i++)
			{
				if (itCar->_status != STATUS_FLAMES && carShooter != itCar)
				{
					if (IsCarHit(itCar, GET_PLAYER_CAR._gun, &shootX, &shootY))
					{
						AddImpact(shootX, shootY);
						DamageCar(itCar, SHOOT_DAMAGES);
						return TRUE;
					}
				}

				itCar++;
			}
		}
	}
	return FALSE;
}

/*****************************************************/
/*													 */
/*	Update gun position								 */
/*												     */
/*****************************************************/
void UpdateCarAndGunPos(SCar* car)
{
	if (car->_gun != GUN_NOTHING)
	{
		/** X position */
		{
			i8 x = 0;
			i8 posRight = GetPosRightX(&car->_roadObj._spriteInfo);

			if (car->_gun & GUN_LEFT)
			{
				x = car->_roadObj._spriteInfo._x - SHOOT_CX;
			}
			else		
			if (car->_gun & GUN_RIGHT)
			{
				x = posRight;
			}
			/** GUN_UP or GUN_DOWN */
			else
			{
				x = posRight - (car->_roadObj._spriteInfo._cx + SHOOT_CX) / 2;
			}
			
			car->_gunSprite._x = x;
		}

		/** Y position */
		{
			car->_gunSprite._y = car->_roadObj._spriteInfo._y + (car->_type == PLAYER_CAR ? 2 : gCarInfos[car->_type]._posWeaponY);
		}
	}
}

/*****************************************************/
/*													 */
/*	Get gunSprite shoot								 */
/*												     */
/*****************************************************/
const u8* GetSpriteShoot(u8 gun, u8 shootState)
{
	u8 orientation = GetShootDirection(gun);

	if (gun & GUN_DOUBLE)
	{
		/** Anti Armor */
		if (gun & GUN_AMMO_ANTIARMOR)
		{
			static u8* const sDShootSprite[2] = { g_shoot_double_4 , g_shoot_double_5 };
			return sDShootSprite[shootState];
		}
		/** Normal gun */
		else
		{
			static u8* const sDShootSprite[2][2] = { { g_shoot_double_0 , g_shoot_double_1 }, { g_shoot_double_2 , g_shoot_double_3 } };
			return sDShootSprite[orientation - 1][shootState];
		}
	}
	else
	{
		static u8* const sSShootSprite[4][2] = { { g_shoot_6 , g_shoot_7 }, {g_shoot_0 , g_shoot_1}, { g_shoot_2 , g_shoot_3 }, { g_shoot_4 , g_shoot_5 } };
		return sSShootSprite[orientation][shootState];
	}
}

/*****************************************************/
/*													 */
/*	Clear shoot										 */
/*												     */
/*****************************************************/
void ClearShoot(SCar *carShooter)
{
	u8 memIndex = !GetVMemIndex();
	SSpriteInfo *gun = &carShooter->_gunSprite;
	
	u8 prevY = gun->_prevPos[memIndex]._y;
	if (prevY != 0)
	{
		i8 prevX = gun->_prevPos[memIndex]._x - 1;
		ClearScreenZone(prevX, prevY, gun->_cx + 1, gun->_cy, BACK_BUFFER);
	}
}

/*****************************************************/
/*													 */
/*	Draw and test shoot								 */
/*												     */
/*****************************************************/
void DrawShoot(SCar *carShooter)
{
	u8 gun = carShooter->_gun;
	if (gun != GUN_NOTHING)
	{
		SSpriteInfo* gunSprite = &carShooter->_gunSprite;

		u8 shootCounter = carShooter->_roadObj._counter--;
		if (shootCounter > 0)
		{
			u8 shootState = shootCounter % 2;

			u8 xGun = gunSprite->_x;
			u8 yGun = gunSprite->_y;
			u8 cyGun = carShooter->_gunSprite._cy;
			u8* shoot = GetSpriteShoot(gun, shootState);
			BOOL drawShoot = TRUE;

			if (shootState == 0)
			{
				if (gun & GUN_HEAVY_GUN)
				{
					if (gRoad._explosions._nbExplosions < NB_EXPLOSIONS_MAX - 1)
						AddExplosion(GetPosRightX(&GET_PLAYER_OBJ._spriteInfo) + Rand(), GET_PLAYER_OBJ._spriteInfo._y - 7 + Rand() * 3);
					else
						drawShoot = FALSE;
				}
				else if (gun & GUN_DOUBLE)
				{
					TestHit(carShooter, xGun, yGun + 5);
					TestHit(carShooter, xGun, gunSprite->_y + carShooter->_roadObj._spriteInfo._cy - 5);
				}
				else
				{
					TestHit(carShooter, xGun, yGun + 4);
				}
			}

			if (drawShoot)
				DrawClippedSprite(shoot, xGun, yGun, SHOOT_CX, cyGun, SPRITE_MASKED, TRUE);
		}
		else
		{
			ClearRoadObject(gunSprite);
			carShooter->_status = STATUS_ACTIVE;
		}
	}
}