//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "toolkit.h"
#include "video.h"
#include "fontsHelper.h"
#include "cars.h"
#include "keyboard.h"
#include "sound.h"
#include "toolkit.h"
#include "player.h"

/*************** Sprites includes *************/
#include ".\gen_src\bitume_logo.h"
#include ".\gen_src\line.h"
#include ".\gen_src\line_color.h"

/*************** Music includes *************/
#include "./gen_src/music.h"

/*************** Extern variables **************/
extern const u8 gGamePalette[];
extern const SCarGraphics gCarStyleset[];
extern u8 gMode;
extern u8 gRoadColor;
extern SRoad gRoad;

/*************** Extern functions **************/
extern void InitPlay();

/*************** Static variables *************/
u8 gFillMode0, gFillMode1;

/*************** Static variables *************/
static i8 sAnimX;
static u8 sCurIndex;
static u8 sWait;
static u8 sLightPos;
static const SCarGraphics* sCarGraph;

static char const sName[3][NAME_LENGTH] =	{ "ASPHALT", "RBLASTER", "UBI SOFT" };
static u16 const sHighScore[3] =			{ 6000, 3000, 1000 };
static u8 sInterrupt = 0;

/*************** Local defines ***************/
#define LOGO_CX					31
#define LOGO_CY					36

#define FILL_Y					LOGO_CY + 2
#define FILL_CY					(SCREEN_CY_BYTE - FILL_Y - 4)
#define FILL_TEXT_Y				91
#define FILL_TEXT_CY			(SCREEN_CY_BYTE - FILL_TEXT_Y)

#define FILL_ROAD_CY			25

#define NB_CARS					10
#define CAR_Y					FILL_Y
#define CAR_X					7

#define TEXT_Y					(FILL_Y + FILL_ROAD_CY + 35)
#define TEXT_X					10
#define OFFSET_Y				14

#define POS_TIRE_Y				78

#define NB_HIGHSCORE			3

/*************** Local Structures ************/
typedef struct TRedefKey
{
	u8* const text;
	u8 key;
} SRedefKey;

/*************** Interrupt handler Timer ******/
static void sInterrupMenuHandler(void)
{
	static u8 const sColor[] = 
	{	
		HW_BRIGHT_WHITE + 0x20, 
		HW_BRIGHT_WHITE + 0x20, 
		HW_BRIGHT_WHITE + 0x20,
		HW_PASTEL_CYAN + 0x20,
		HW_PASTEL_YELLOW + 0x20,
		HW_PASTEL_YELLOW + 0x20
	};

	if (gMode & MENU)
	{
		cpct_setVideoMode(sInterrupt == 2 ? 0 : 1);
		cpct_setPALColour(2, sColor[sInterrupt]);
	}
	else
		SetAllPaletteBlack();
	
	if (sInterrupt == 4 && sWait != 255)
		PlaySound();

	if (++sInterrupt == 6)
	{
		sInterrupt = 0;
		cpct_scanKeyboard_if();
	}
}

/*****************************************************/
/*													 */
/*	Draw arrows										 */
/*												     */
/*****************************************************/
void DrawArrows(u8 max)
{
	u8 i;
	for (i = 0; i < max; i += G_LINE_W)
		DrawSpriteType(g_line, GetScreenPtr(i, POS_TIRE_Y), G_LINE_W, G_LINE_H, SPRITE_OPAQUE);
}

/*****************************************************/
/*													 */
/*	Draw score										 */
/*												     */
/*****************************************************/
void DrawScore()
{
	u8 i;
	for (i = 0; i < NB_HIGHSCORE; i++)
	{
		u8 y = TEXT_Y + 4 + i * OFFSET_Y;
		PrintInt(TEXT_X, y, i + 1, 0);
		PrintStr(TEXT_X + 10, y, sName[i]);
		PrintInt(TEXT_X + 50, y, sHighScore[i], 0);
	}
}

/*****************************************************/
/*													 */
/*	Draw score										 */
/*												     */
/*****************************************************/
void DrawCredit()
{
	PrintStr(TEXT_X + 9, TEXT_Y + 58, "GAME BY ARNAUD BOUCHE");
	PrintStr(TEXT_X + 15, TEXT_Y + 72, "USING CPCTELERA");
	PrintStr(TEXT_X + 4, TEXT_Y + 88, "PRESS -R- TO REDEFINE KEYS");
}

/*****************************************************/
/*													 */
/*	Draw text										 */
/*												     */
/*****************************************************/
void DrawText()
{
	FillWidthScreen(SCREEN_BUFFER, FILL_TEXT_Y, FILL_TEXT_CY, gFillMode1);

	DrawScore();
	DrawCredit();
}

/*****************************************************/
/*													 */
/*	Enter highscore									 */
/*												     */
/*****************************************************/
void EnterHighScore()
{
	u16 money = GET_PLAYER._money;
	u16* highScore = sHighScore;

	u8 i;
	for (i = 0; i < 3; i++)
	{
		if (money > highScore[i])
		{
			cpct_memcpy((u8*)sName[i], GET_PLAYER._name, NAME_LENGTH);
			highScore[i] = money;
			break;
		}
	}
}

/*****************************************************/
/*													 */
/*	Draw menu										 */
/*												     */
/*****************************************************/
void InitMenu(BOOL endGame)
{
	u8 *tempBackBuffer = WinCpcTelera_GetMemPtr(FREE_AREA_MEM);
	gMode = NOTHING;
	sWait = 255;
	sLightPos = 0;

	/** Set fonts menu */
	SetDisplay(VIDEO_MEM);	
	SetBufferFont(SCREEN_BUFFER);
	SetGameFont(FALSE);

	/** Set menu interrupt */
	sInterrupt = 0;
	SetInterruptHandler(sInterrupMenuHandler);

	/** Extract in backBuffer memory */
	Decompress(G_bitume_logo, tempBackBuffer);

	/** Draw to videoBuffer board */
	FillWidthScreen(SCREEN_BUFFER, 0, LOGO_CY + 2, gFillMode1);

	/** Draw logo */
	cpct_drawSprite(tempBackBuffer, cpctm_screenPtr(CPCT_VMEM_START, (SCREEN_CX_BYTE - LOGO_CX) / 2, 0), LOGO_CX, LOGO_CY);
	
	/** Fill Road background*/
	FillWidthScreen(SCREEN_BUFFER, FILL_Y, FILL_CY, gFillMode0);

	/** Set menu */
	gMode = MENU;

	if (endGame)
		EnterHighScore();

	/** Draw Text */
	DrawText();

	/** Draw line */
	DrawArrows(SCREEN_CX_BYTE);

	/** Decompress music */
	Decompress(G_music, WinCpcTelera_GetMemPtr(MUSIC_ADDRESS));
	InitMusic();
	sWait = 0;
	
	/** Set menu palette */
	SetFullPalette(gGamePalette);
	cpct_setBorder(HW_WHITE);

	/** Init Drawing Cars */
	sAnimX = -(i8)gCarStyleset[0]._cx;
	sCarGraph = &gCarStyleset[0];
	sCurIndex = 0;
}

/*****************************************************/
/*													 */
/*	Draw key to redefine							 */
/*												     */
/*****************************************************/
void DrawRedefineKey(u8 y, const SRedefKey* rkey)
{
	PrintStr(TEXT_X + 9, TEXT_Y + y + 5, rkey->text);
	RedefineKey(rkey->key);
}

/*****************************************************/
/*													 */
/*	RedifinesKeys									 */
/*												     */
/*****************************************************/
void RedefineKeys()
{
	static const SRedefKey rkeys[] = {
										{ "UP", KEY_FOR_UP },
										{ "DOWN", KEY_FOR_DOWN },
										{ "LEFT", KEY_FOR_LEFT },
										{ "RIGHT", KEY_FOR_RIGHT},
										{ "FIRE", KEY_FOR_FIRE},
										{ "CHANGE WEAPON", KEY_FOR_WEAPON } 
									};

	/** Clear 'R' pressed */
	while (cpct_isKeyPressed(Key_R));

	/** Fill Text background*/
	FillWidthScreen(SCREEN_BUFFER, FILL_TEXT_Y, FILL_TEXT_CY, gFillMode1);

	/** Draw keys to define */
	{
		const SRedefKey* rkey = rkeys;

		u8 i;
		for (i = 0; i < sizeof(rkeys) / sizeof(SRedefKey); i++)
			DrawRedefineKey(i * OFFSET_Y + 4, rkey++);
	}

	/** Draw Text */
	DrawText();
}

/*****************************************************/
/*													 */
/*	Draw menu										 */
/*												     */
/*****************************************************/
void Menu()
{
	if (cpct_isKeyPressed(Key_R))
		RedefineKeys();

	if (ReadKeyboardInput() == VKEY_ACTION)
	{
		cpct_akp_stop();
		InitPlay();	
	}
	else
	{
		u8 cx = sCarGraph->_cx;
		u8 cy = sCarGraph->_cy;

		cpct_waitVSYNC();
		if (sWait-- == 0)
		{
			sWait = 2;

			if (sAnimX >= 0)
				cpct_drawSolidBox(GetScreenPtr(sAnimX, CAR_Y + 10), gFillMode0, 1, cy);

			DrawSpriteType(g_line_color, GetScreenPtr(sLightPos += 2, POS_TIRE_Y), G_LINE_COLOR_W, G_LINE_COLOR_H, SPRITE_OPAQUE);
			if (sLightPos >= SCREEN_CX_BYTE - 4)
				sLightPos = 0;

			DrawArrows(sLightPos == 0 ? SCREEN_CX_BYTE : sLightPos);
			DrawClippedSprite(sCarGraph->_sprite, ++sAnimX, CAR_Y + 10, cx, cy, SPRITE_OPAQUE, FALSE);

			if (sAnimX == SCREEN_CX_BYTE)
			{
				if (++sCurIndex == NB_CARS)
					sCurIndex = 0;

				sAnimX = -(i8)(cx + 1);
				sCarGraph = &gCarStyleset[sCurIndex];
			}
		}
	}
}
