//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <stdio.h>

/*************** Game includes ****************/
#include "structures.h"
#include "sound.h"
#include "video.h"
#include "road.h"
#include "player.h"
#include "mines.h"
#include "missiles.h"
#include "cars.h"
#include "gun.h"
#include "fontsHelper.h"
#include "toolkit.h"
#include "explosions.h"

/*************** Local defines ****************/
#define TARGET_WINDOW		8
#define ENTER_ZONE_LEFT		15
#define ENTER_ZONE_RIGHT	(SCREEN_CX_BYTE - 5)
#define IMPACT_WAIT			10
#define CHANGE_DIR_WAIT		12

/*************** Local functions ***************/
void MoveTrack(void* pCar);
void MoveOverTakeFromLeft(void* pCar);
void IsTracking(SCar* car, BOOL fromRight);

/*****************************************************/
/*													 */
/*	Change move										 */
/*												     */
/*****************************************************/
void ChangeMove(SCar* car, TMove move)
{
	car->Move = move;
	ClearShoot(car);
}

/*****************************************************/
/*													 */
/*	Car shoot on player								 */
/*												     */
/*****************************************************/
void ShootOnPlayer(SCar* car)
{
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;

	/** In front of player */
	if (IN_SCREEN(GetPosRightX(sprite), sprite->_x) && TestHorizontalCollision(sprite, &GET_PLAYER_CAR._roadObj._spriteInfo))
	{
		u8 weapon = car->_weapon;
		if (weapon != NO_WEAPON)
		{
			if (weapon == MISSILE)				
				UpdateMissileCounter(car);
			else
				UpdateMineCounter(weapon, car);
		}
		else
			UpdateShootCounter(car);
	}
}

/*****************************************************/
/*													 */
/*	Test if car collide								 */
/*												     */
/*****************************************************/
SCar* TestCollisionCars(SCar* car)
{
	SCar* itCar = gRoad._cars._car;

	u8 i;
	for (i = 0; i < gRoad._cars._nbCars; i++)
	{
		if (car != itCar)
		{
			SSpriteInfo* sprite =  &itCar->_roadObj._spriteInfo;
			if (itCar->_status != STATUS_NOTHING && GetPosRightX(sprite) >= 0)
			{
				if (TestCollision(&itCar->_roadObj._spriteInfo, &car->_roadObj._spriteInfo))
					return itCar;
			}
		}
		itCar++;
	}
	return NULL;
}

/*****************************************************/
/*													 */
/*	Test if cars collide 							 */
/*												     */
/*****************************************************/
SCar* TestCollisionCar(SCar* car)
{
	if (car->_status != STATUS_NOTHING)
	{
		SSpriteInfo* sprite = &car->_roadObj._spriteInfo;
		if (GetPosRightX(sprite) > 0 && TestCollision(sprite, &GET_PLAYER_OBJ._spriteInfo))
			return GET_PLAYER_CAR_PTR;

		return TestCollisionCars(car);

	}
	return NULL;
}

/*****************************************************/
/*													 */
/*	Test if collide X then Y						 */
/*												     */
/*****************************************************/
SCar* XandYTestCollisionCar(SCar* car, i8 x, i8 y)
{
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;
	SCar* collideCar = NULL;
	
	/** Collision X */
	if (x != 0)
	{
		i8 prevPosX = sprite->_x;

		sprite->_x += x;
		collideCar = TestCollisionCar(car);
		if (collideCar != NULL)
		{
			sprite->_x = prevPosX;
		}
	}

	/** Collision Y */
	if (y != 0)
	{
		SCar* collideCarY = NULL;
		u8 prevPosY = sprite->_y;

		sprite->_y += y;

		collideCarY = TestCollisionCar(car);
		if (collideCarY != NULL)
		{
			sprite->_y = prevPosY;
			collideCar = collideCarY;
		}

		sprite->_y = GetLimitSpriteY(sprite);
	}

	return collideCar;
}

/*****************************************************/
/*													 */
/*	Track player position Y							 */
/*												     */
/*****************************************************/
i8 GetTrackPosY(SCar* car, u8 window)
{
	u8 yCenterCar = GetSpriteCenterY(&car->_roadObj._spriteInfo);
	u8 yCenterPlayer = GetSpriteCenterY(&GET_PLAYER_CAR._roadObj._spriteInfo);
	u8 speed = car->_speed * 2;

	if (yCenterCar < yCenterPlayer - window)
		return speed;
	else
	if (yCenterCar > yCenterPlayer + window)
		return -speed;

	return 0;
}

/*****************************************************/
/*													 */
/*	Move escape										 */
/*												     */
/*****************************************************/
void MoveEscape(void* pCar)
{
	SCar* car = (SCar*)pCar;
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;
	u8 speed = car->_speed;

	if (car->_gun & GUN_LEFT || car->_gun == GUN_NOTHING)
	{
		i8 y = 0;
		if (car->_waitTime == 0)
		{
			y = GetTrackPosY(car, TARGET_WINDOW);
			car->_waitTime = Rand() & 0x01;
		}
		else
			car->_waitTime--;
		
		XandYTestCollisionCar(car, speed, y);
		ShootOnPlayer(car);
	}
	else
	{
		u8 ySpeed = speed * 2;

		if (GetPosRightX(sprite) > GetPosRightX(&GET_PLAYER_OBJ._spriteInfo))
		{
			if (car->_type == EN_CAR_1)
				car->_gun = GUN_DOUBLE_LEFT;
			else
			if (car->_type == EN_MBIKE_9_R)
			{
				car->_gun = GUN_NOTHING;
				car->_weapon = NAILS;
			}
		}

		if (TestHorizontalCollision(sprite, &GET_PLAYER_CAR._roadObj._spriteInfo))
		{
			i8 y = 0;
			if ((u8)GetTrackPosY(car, GET_PLAYER_CAR._roadObj._spriteInfo._cy + 1) == Rand() / 2)
			{
				u8 yCenterCar = GetSpriteCenterY(&car->_roadObj._spriteInfo);
				u8 yCenterPlayer = GetSpriteCenterY(&GET_PLAYER_CAR._roadObj._spriteInfo);
				
				y = (yCenterCar > yCenterPlayer) ? ySpeed : -ySpeed;
			}

			XandYTestCollisionCar(car, speed, y);
		}
		else
		{
			if (XandYTestCollisionCar(car, speed, 0) != NULL)
			{
				if ((XandYTestCollisionCar(car, 0, ySpeed) != NULL))
				{
					XandYTestCollisionCar(car, 0, -ySpeed);
				}
			}
		}
	}
}

/*****************************************************/
/*													 */
/*	Car escape										 */
/*												     */
/*****************************************************/
void CarExit(SCar* car)
{
	car->_status = STATUS_EXIT;
	ChangeMove(car, MoveEscape);
}

/*****************************************************/
/*													 */
/*	Test if car escape								 */
/*												     */
/*****************************************************/
void IsCarEscape(SCar* car, SSpriteInfo* sprite)
{
	/** Enemy escape to the right */
	if ((car->_nbShoot == 0 && sprite->_x < ENTER_ZONE_LEFT) || (GetPosRightX(sprite) > GET_PLAYER_CAR._roadObj._spriteInfo._x && (car->_gun & GUN_RIGHT)))
		CarExit(car);
}

/*****************************************************/
/*													 */
/*	Test if shoot vertical escape					 */
/*												     */
/*****************************************************/
void ShootVertical(SCar* car)
{
	if (TestVerticalCollision(&car->_roadObj._spriteInfo, &GET_PLAYER_CAR._roadObj._spriteInfo))
		UpdateShootCounter(car);
}

/*****************************************************/
/*													 */
/*	Move Car linear	to left							 */
/*												     */
/*****************************************************/
void MoveLinearLeft(void* pCar)
{
	SCar* car = (SCar*)pCar;
	car->_roadObj._spriteInfo._x -= SPEED_SCROLL;
}

/*****************************************************/
/*													 */
/*	Move collide player								 */
/*												     */
/*****************************************************/
void MoveToCollide(void* pCar)
{
	SCar* carCollide;
	SCar* car = (SCar*)pCar;
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;

	if (car->_waitTime == 0)
	{
		i8 moveY = GetTrackPosY(car, TARGET_WINDOW);
		if (moveY != 0)
			sprite->_y += moveY;

		car->_waitTime = Rand() & 0x01;
	}
	else
		car->_waitTime--;

	sprite->_x += car->_speed;
	carCollide = TestCollisionCar(car);
	if (carCollide)
	{
		sprite->_x -= SPEED_SCROLL;
		DamageCar(carCollide, EXPLOSION_CAR);
		DamageCar(car, EXPLOSION_CAR);
	}

	if (GetPosRightX(sprite) > GET_PLAYER_CAR._roadObj._spriteInfo._x)
		CarExit(car);
}

/*****************************************************/
/*													 */
/*	Move Car linear									 */
/*												     */
/*****************************************************/
void MoveLinearToRight(void* pCar)
{
	SCar* car = (SCar*)pCar;
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;
	XandYTestCollisionCar(car, car->_speed, 0);

	if (sprite->_x > SCREEN_CX_BYTE - 1)
		car->_status = STATUS_EXIT;
	else
		ShootVertical(car);
}

/*****************************************************/
/*													 */
/*	Move Car destroyed								 */
/*												     */
/*****************************************************/
void MoveCarDestroyed(void* pCar)
{
 	SCar* car = (SCar*)pCar;
	SCar* collidedCar = XandYTestCollisionCar(car, -SPEED_SCROLL, 0);

	if (collidedCar != NULL && !(collidedCar->_status & STATUS_FLAMES))
	{
		if (car->_type != EN_MINE_FIELD)
		{
			if (car->_type == EN_BARREL)
			{
				AddExplosionObject(&car->_roadObj._spriteInfo);
				DeleteCar(car);
			}
			else
				DamageCar(collidedCar, FIRE_DAMAGES);

			PlaySFX(FX_IMPACT, FX_IMPACT_CHANNEL);
		}
	}
}

/*****************************************************/
/*													 */
/*	Enter Car from left								 */
/*												     */
/*****************************************************/
void MoveEnterFromLeft(void* pCar)
{
	SCar* car = (SCar*)pCar;

	XandYTestCollisionCar(car, car->_speed, GetTrackPosY(car, TARGET_WINDOW));
	ShootOnPlayer(car);

	/** Ready to track */
	IsTracking(car, FALSE);
}

/*****************************************************/
/*													 */
/*	Enter Car from right							 */
/*												     */
/*****************************************************/
static void MoveEnterFromRight(SCar* car)
{
	XandYTestCollisionCar(car, -car->_speed, GetTrackPosY(car, TARGET_WINDOW));
	ShootOnPlayer(car);
}

/*****************************************************/
/*													 */
/*	Enter Car from right to front					 */
/*												     */
/*****************************************************/
void MoveEnterFromRightToFront(void* pCar)
{
	SCar* car = (SCar*)pCar;
	MoveEnterFromRight(car);
	
	/** Ready to track */
	IsTracking(car, TRUE);
}

/*****************************************************/
/*													 */
/*	Enter Car from right to rear					 */
/*												     */
/*****************************************************/
void MoveEnterFromRightToRear(void* pCar)
{
	SCar* car = (SCar*)pCar;
	XandYTestCollisionCar(car, -SPEED_SCROLL, 0);

	/** Ready to track */
	IsTracking(car,  TRUE);
}

/*****************************************************/
/*													 */
/*	Is car ready to track player					 */
/*												     */
/*****************************************************/
void IsTracking(SCar* car, BOOL fromRight)
{
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;
	u8 offset = Rand();
	u8 posRight = GetPosRightX(sprite) + offset;

	if ((sprite->_x + offset > ENTER_ZONE_LEFT && !fromRight) || ((car->_gun & GUN_RIGHT && posRight < ENTER_ZONE_LEFT) || ((car->_gun == GUN_NOTHING || car->_gun & GUN_LEFT) && posRight < ENTER_ZONE_RIGHT)) && fromRight)
	{
		car->_status = STATUS_ACTIVE;
		ChangeMove(car, MoveTrack);
	}
}

/*****************************************************/
/*													 */
/*	Move track player								 */
/*												     */
/*****************************************************/
void MoveUpAndDown(void* pCar)
{
	SCar* car = (SCar*)pCar;
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;

	u8 prevPosY = sprite->_y;
	i8 dir = car->_waitTime;
	u8 speed = car->_speed * 2;

	sprite->_y += dir;
	if (TestCollisionCar(car))
	{
		car->_waitTime = (dir > 0) ? -speed : speed;
		sprite->_y = prevPosY;
	}

	ShootOnPlayer(car);

	if (dir == 0 || sprite->_y < LIMIT_UP)
		car->_waitTime = speed;
	else if (GetPosRightY(sprite) > START_DOWN_CAR - 5)
		car->_waitTime = -speed;
}

/*****************************************************/
/*													 */
/*	Move track player								 */
/*												     */
/*****************************************************/
void MoveTrack(void* pCar)
{
	SCar* car = (SCar*)pCar;
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;
	i8 moveY = GetTrackPosY(car, TARGET_WINDOW);

	/**  Wait before next move */
	if (car->_waitTime == 0 && moveY == 0)
		car->_waitTime = CHANGE_DIR_WAIT;

	/** In front of player */
	ShootOnPlayer(car);

	/** Move car */
	if (moveY != 0 && car->_waitTime == 0)
	{
		u8 prevPosY = sprite->_y;
		sprite->_y += moveY;

		if (TestCollisionCar(car))
			sprite->_y = prevPosY;
	}

	if (car->_waitTime > 0)
		car->_waitTime--;

	IsCarEscape(car, sprite);
}

/*****************************************************/
/*													 */
/*	Move avoid Left									 */
/*												     */
/*****************************************************/
void MoveOverTakeFromLeft(void* pCar)
{
	SCar* car = (SCar*)pCar;
	SSpriteInfo* sprite = &car->_roadObj._spriteInfo;
	SSpriteInfo* spritePlayer = &GET_PLAYER_CAR._roadObj._spriteInfo;

	if (sprite->_x < spritePlayer->_x)
	{
		if (TestHorizontalCollision(sprite, spritePlayer))
			XandYTestCollisionCar(car, car->_speed, car->_speed);
		else
			sprite->_x += car->_speed;
	}
	else
	{
		if (sprite->_x > GetPosRightX(spritePlayer) + 10)
			ChangeMove(car, MoveUpAndDown);
		else
			XandYTestCollisionCar(car, car->_speed, GetTrackPosY(car, TARGET_WINDOW));
	}
}