//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>

/*************** Game includes ****************/
#include "structures.h"
#include "video.h"
#include "sound.h"
#include "menus.h"
#include "road.h"
#include "game.h"
#include "cars.h"
#include "mines.h"
#include "missiles.h"
#include "player.h"
#include "move.h"
#include "fontsHelper.h"
#include "gun.h"
#include "board.h"
#include "toolkit.h"

/*************** Sprites includes *************/
#include ".\gen_src\arrow.h"
#include ".\gen_src\shoot.h"
#include ".\gen_src\shoot_double.h"
#include ".\gen_src\flame.h"

/*************** Extern functions *************/
extern u8 ReadKeyboardInput();

/*************** Local functions ***************/
void PlayerCarRepaired();
void UpdateBonusCounter();

/*************** Local defines ****************/
#define ARROW_X				20
#define ARROW_Y				BOARD_CY-7
#define ARROW_CX			G_ARROW_0_W
#define ARROW_CY			G_ARROW_0_H

#define FLAMES_CX			G_FLAME_00_W
#define FLAMES_CY			G_FLAME_00_H

#define TIME_RELOAD_MINE	30
#define TIME_RELOAD_MISS	2

#define REPAIR				20
#define LIMIT_X_EXIT		-60

/*************** Extern variables *************/
u8 gCarSprite[PLAYER_CX * PLAYER_CY];

/*****************************************************/
/*													 */
/*	Default player									 */
/*												     */
/*****************************************************/
void DefaultPlayer()
{
	GET_PLAYER_CAR._speed = 1;
	GET_PLAYER_CAR._gun = GUN_DOUBLE_RIGHT;
	GET_PLAYER_CAR._protection = 0;

	GET_PLAYER._bonus._id = BONUS_NOTHING;
	GET_PLAYER._bonus._counter = 1;
}

/*****************************************************/
/*													 */
/*	Enter player in road							 */
/*												     */
/*****************************************************/
void EnterPlayer()
{
	SSpriteInfo* spriteInfo = &GET_PLAYER_OBJ._spriteInfo;
	spriteInfo->_x = PLAYER_ENTER_X;
	spriteInfo->_y = PLAYER_START_Y;

	UpdateCarAndGunPos(GET_PLAYER_CAR_PTR);

	GET_PLAYER_CAR._status = STATUS_ENTER;
	GET_PLAYER_CAR._type = PLAYER_CAR;

	GET_PLAYER_OBJ._counter = NO_SET_I8;
	GET_PLAYER_CAR._flames = NO_FLAMES;

	/** Player destroyed or new game */
	if (GET_PLAYER_CAR._armor == 0)
		GET_PLAYER_CAR._armor = MAX_ARMOR;
	
	DefaultPlayer();

	GET_PLAYER._mines = 1;
	GET_PLAYER._missiles = 1;

	UpdateInfoAmmo();
	UpdateInfoArmor();
	UpdateInfoLife();
	UpdateInfoBonus();

	INIT_MINE;
}

/*****************************************************/
/*													 */
/*	Init player	for new game					 	*/
/*												     */
/*****************************************************/
void InitPlayerNewGame()
{
	EnterPlayer();
	
	GET_PLAYER._weaponSelect = WEAPON_GUN;
	GET_PLAYER._heat = 0;
	GET_PLAYER._mines = START_MINES;
	GET_PLAYER._missiles = START_MISSILES;

	GET_PLAYER_CAR._armor = MAX_ARMOR;

	GET_PLAYER._life = GET_PLAYER._cheatMode ? CHEAT_LIFES : START_LIFES;
	GET_PLAYER._money = 0;
	GET_PLAYER._bonus._enterBonus = 0;
}

/*****************************************************/
/*													 */
/*	Clear player background							 */
/*												     */
/*****************************************************/
void ClearPlayer()
{
	static BOOL update = FALSE;

	/** Clear flames */
	if (GET_PLAYER_CAR._status & STATUS_FLAMES)
		ClearFlamesBackground(GET_PLAYER_CAR_PTR);

	/** Clear player car */
	DrawRoadObjectBackground(&GET_PLAYER_OBJ._spriteInfo);
	DrawRoadObjectBackground(&GET_PLAYER_CAR._gunSprite);
}

/*****************************************************/
/*													 */
/*	Draw car at current position					 */
/*												     */
/*****************************************************/
void DrawPlayer()
{
	/** Draw player car */
	DrawClippedRoadObjectOpaque(&GET_PLAYER_OBJ);
	
	/** Draw flames */
	if (GET_PLAYER_CAR._status & STATUS_FLAMES)
		DrawFlames(GET_PLAYER_CAR_PTR);

	/** Draw shoot */
	else if (GET_PLAYER_CAR._status & STATUS_SHOOT)
		DrawShoot(GET_PLAYER_CAR_PTR);
}

/*****************************************************/
/*													 */
/*	Update player car								 */
/*												     */
/*****************************************************/
void UpdatePlayer()
{
	/** Flames */
	if (GET_PLAYER_CAR._status & STATUS_FLAMES)
	{
		GET_PLAYER_CAR._flames++;
		return;
	}

	/** Played destroyed */
	if (GET_PLAYER_CAR._armor == 0 && !(GET_PLAYER_CAR._status & STATUS_FLAMES))
		CarDestroyed(GET_PLAYER_CAR_PTR);

	/** Reload mine */
	if (GET_PLAYER._mineReload > 0)
		GET_PLAYER._mineReload--;

	/** Reload missile */
	if (GET_PLAYER._missileReload > 0)
		GET_PLAYER._missileReload--;
	
	/** Update bonus */
	if (GET_PLAYER._bonus._id != BONUS_NOTHING)
	{
		if (GET_PLAYER._bonus._id == BONUS_REPAIR)
			PlayerCarRepaired();

		UpdateBonusCounter();
	}

	/* Update gun heat */
	UpdateHeat();
}

/*****************************************************/
/*													 */
/*	Car destroyed									 */
/*												     */
/*****************************************************/
void PlayerCarDestroyed()
{
	SCar* itCar = gRoad._cars._car;

	u8 i;
	for (i = 0; i < gRoad._cars._nbCars; i++)
	{
		ChangeMove(itCar, MoveLinearLeft);
		if (!(itCar->_status & STATUS_FLAMES))
			itCar->_status = STATUS_EXIT;

		ClearShoot(itCar);
		itCar++;
	}

	UpdateInfoArmor();
}

/*****************************************************/
/*													 */
/*	Auto move player car							 */
/*												     */
/*****************************************************/
BOOL AutoMove()
{
	SSpriteInfo* sprite = &GET_PLAYER_OBJ._spriteInfo;

	u8 status = GET_PLAYER_CAR._status;
	if (status & STATUS_FLAMES)
	{
		i8 posX = GetPosRightX(sprite);
		if (posX > LIMIT_X_EXIT)
			sprite->_x -= SPEED_SCROLL;

		if (posX <= LIMIT_X_EXIT && IsRoundFinished())
		{
			if (GET_PLAYER._life > 1)
			{
				GET_PLAYER._life--;
				EnterPlayer();
			}		
			else
			{
				/* Game over */
				InitMenu(TRUE);
			}
		}		
		return TRUE;
	}
	else
	if (status & (STATUS_ENTER | STATUS_END_LEVEL))
	{
		if ((sprite->_x == PLAYER_START_X) && (status & STATUS_ENTER))
		{
			GET_PLAYER_CAR._status = STATUS_ACTIVE;
			StartRound();
		}
		else if ((sprite->_x > SCREEN_CX_BYTE) && (status & STATUS_END_LEVEL))
		{
			if (!IsGameWin())
			{
				/** Level repair free */
				u8 repair = GET_PLAYER_CAR._armor + REPAIR;
				GET_PLAYER_CAR._armor = (repair > MAX_ARMOR) ? MAX_ARMOR : repair;
				UpdateInfoArmor();

				GET_PLAYER_CAR._status = STATUS_ENTER;
				sprite->_x = PLAYER_ENTER_X;
				sprite->_y = PLAYER_START_Y;

				InitRoad();
				cpct_memcpy(GetOriginBackScreenPtr(), GetOriginScreenPtr(), SCREEN_SIZE);
			}
			else
				EndGame();
		}
		else
			sprite->_x += SPEED_SCROLL;

		return TRUE;
	}

	return FALSE;
}

/*****************************************************/
/*													 */
/*	Change weapon selection							 */
/*												     */
/*****************************************************/
void ChangeWeapon()
{
	if (++GET_PLAYER._weaponSelect == WEAPON_NB)
		GET_PLAYER._weaponSelect = WEAPON_GUN;
		
	UpdateInfoWeapon();

	PlaySFX(FX_SELECT, FX_SELECT_CHANNEL);
}

/*****************************************************/
/*													 */
/*	Bonus Update counter							 */
/*												     */
/*****************************************************/
void UpdateBonusCounter()
{
	if (GET_PLAYER._bonus._counter == 0)
	{
		GET_PLAYER._bonus._id = BONUS_NOTHING;
		UpdateInfoBonus();
	}

	UpdateCounterBonus();
}

/*****************************************************/
/*													 */
/*	Bonus Repair car								 */
/*												     */
/*****************************************************/
void PlayerCarRepaired()
{
	if (GET_PLAYER_CAR._armor < MAX_ARMOR)
	{
		PlayerCarDecrBonus(DECR_BONUS_REPAIR);
		GET_PLAYER_CAR._armor++;
		UpdateInfoArmor();	
	}
}

/*****************************************************/
/*													 */
/*	Bonus car decrement counter						 */
/*												     */
/*****************************************************/
void PlayerCarDecrBonus(u8 decrBonus)
{
	DecrValue(&GET_PLAYER._bonus._counter, decrBonus);
}

/*****************************************************/
/*													 */
/*	Use selected weapon								 */
/*												     */
/*****************************************************/
void UseWeapon()
{
	/** Use guns */
	if (GET_PLAYER_CAR._waitTime == 0 && GET_PLAYER._weaponSelect == WEAPON_GUN)
	{
		UpdateShoot(&GET_PLAYER_CAR);
		PlaySFX(FX_SHOOT, FX_SHOOT_CHANNEL);

		if (GET_PLAYER._bonus._id == BONUS_AMMO)
		{
			PlayerCarDecrBonus(DECR_BONUS_AMMO);
			if (GET_PLAYER._bonus._counter == 0)
				GET_PLAYER_CAR._gun = GUN_DOUBLE_RIGHT;
		}
	}
	/** Drop mine */
	else if (GET_PLAYER._weaponSelect == WEAPON_MINE && GET_PLAYER._mines != 0 && gRoad._mines._nbMinesDrop != NB_MINES_DROP_MAX)
	{
		if (GET_PLAYER._mineReload == 0)
		{
			GET_PLAYER._mineReload = TIME_RELOAD_MINE;
			GET_PLAYER._mines--;
			DropMine(MINE, GET_PLAYER_CAR_PTR);
			UpdateInfoAmmo();
		}
	}
	/** Launch missile */
	else if (GET_PLAYER._weaponSelect == WEAPON_MISSILE && GET_PLAYER._missiles != 0 && gRoad._missiles._nbMissilesLaunch != NB_MISS_LAUNCH_MAX)
	{
		if (GET_PLAYER._missileReload == 0)
		{
			GET_PLAYER._missileReload = TIME_RELOAD_MISS;
			GET_PLAYER._missiles--;
			LaunchMissile(GET_PLAYER_CAR_PTR);
			UpdateInfoAmmo();
		}
	}
}

/*****************************************************/
/*													 */
/*	Is auto move car player							 */
/*												     */
/*****************************************************/
BOOL IsAutoMove()
{
	return GET_PLAYER_CAR._status & (STATUS_FLAMES | STATUS_ENTER | STATUS_END_LEVEL);
}

/*****************************************************/
/*													 */
/*	Move player position							 */
/*												     */
/*****************************************************/
static i8 MovePlayer(u8 key, u8 keyMaskInc, u8 keyMaskDec, u8 speed)
{
	if (key & keyMaskInc)
		return speed;
	else
	if (key & keyMaskDec)
		return -speed;

	return 0;
}

/*****************************************************/
/*													 */
/*	Update input									 */
/*												     */
/*****************************************************/
void UpdatePlayerInput()
{
	u8 curKey = ReadKeyboardInput();
	if (curKey == VKEY_ESCAPE)
	{
		InitMenu(FALSE);
	}
	else
	{
		SSpriteInfo* sprite = &GET_PLAYER_OBJ._spriteInfo;
		StoreCurrentPos(&GET_PLAYER_CAR._gunSprite);
		StoreCurrentPos(sprite);

		if (curKey != VKEY_NOTHING || IsAutoMove())
		{
			if (AutoMove() == FALSE)
			{
				i8 deltaMove = MovePlayer(curKey, VKEY_RIGHT, VKEY_LEFT, GET_PLAYER_CAR._speed);
				if (deltaMove != 0)
				{
					i8 prevX = sprite->_x;
					sprite->_x += deltaMove;
					sprite->_x = TestCollisionCars(GET_PLAYER_CAR_PTR) ? prevX : GetLimitX(sprite->_x);
				}

				deltaMove = MovePlayer(curKey, VKEY_DOWN, VKEY_UP, GET_PLAYER_CAR._speed*2);
				if (deltaMove != 0)
				{
					u8 prevY = sprite->_y;
					sprite->_y += deltaMove;
					sprite->_y = TestCollisionCars(GET_PLAYER_CAR_PTR) ? prevY : GetLimitSpriteY(sprite);
				}

				if (curKey & VKEY_SELECTION)
					ChangeWeapon();
				else
				if (curKey & VKEY_ACTION)
					UseWeapon();
			}

			UpdateCarAndGunPos(&GET_PLAYER_CAR);
		}
	}
}
