//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Bitume 
//  Copyright (C) 2017 Arnaud Bouche (Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>
#include <stdio.h>
#include "wincpctelera_helper.h"

/*************** Game includes ****************/
#include "structures.h"
#include "toolkit.h"
#include "fontsHelper.h"
#include "video.h"
#include "road.h"

/*************** static variables **************/
static BOOL sVMem;

/*************** Global constants **************/
const u8 gGamePalette[] =
{
	/* 0 */ HW_BLACK, 			// 0
	/* 1 */ HW_WHITE, 			// 13	
	/* 2 */ HW_BRIGHT_WHITE,	// 26
	/* 3 */ HW_SKY_BLUE,		// 11
	/* 4 */ HW_BRIGHT_BLUE,		// 2
	/* 5 */ HW_RED,				// 3
	/* 6 */ HW_BRIGHT_RED, 		// 6
	/* 7 */ HW_BRIGHT_MAGENTA, 	// 8
	/* 8 */ HW_GREEN, 			// 9
	/* 9 */ HW_BLUE, 			// 1
	/*10 */ HW_YELLOW, 			// 12
	/*11 */ HW_ORANGE, 			// 15
	/*12 */ HW_BRIGHT_GREEN, 	// 18
	/*13 */ HW_BRIGHT_YELLOW,	// 24 
	/*14 */ HW_MAGENTA,			// 4
	/*15 */ HW_PINK				// 16
};

const u8 gBoardPalette[] =
{
	/* 0 */ 0x54, // HW_BLACK			- 0
	/* 1 */ 0x40, // HW_WHITE			- 13
	/* 2 */ 0x4b, // HW_BRIGHT_WHITE	- 26
	/* 3 */ 0x57, // HW_SKY_BLUE		- 11
	/* 4 */ 0x55, // HW_BRIGHT_BLUE		- 2
	/* 5 */ 0x5c, // HW_RED				- 3
	/* 6 */ 0x4c, // HW_BRIGHT_RED		- 6
	/* 7 */ 0x4d, // HW_BRIGHT_MAGENTA	- 8
	/* 8 */ 0x56, // HW_GREEN			- 9
	/* 9 */ 0x44, // HW_BLUE			- 1
	/*10 */ 0x5e, // HW_YELLOW			- 12
	/*11 */ 0x4e, // HW_ORANGE			- 15
	/*12 */ 0x52, // HW_BRIGHT_GREEN	- 18
	/*13 */ 0x4a, // HW_BRIGHT_YELLOW	- 24
	/*14 */ 0x42, // HW_SEA_GREEN		- 19
	/*15 */ 0x53  // HW_BRIGHT_CYAN		- 20
};

/*************** Transparent Mask Table ********/
cpctm_createTransparentMaskTable(gMaskTable, MASKTABLE, M0, 0);

/*****************************************************/
/*													 */
/*	Set full palette								 */
/*												     */
/*****************************************************/
void SetFullPalette(u8* palette)
{
	cpct_setPalette(palette, 16);
}

/*****************************************************/
/*													 */
/*	Set palette all same color						 */
/*												     */
/*****************************************************/
void SetAllPaletteBlack()
{
	u8 i;
	for (i = 0; i < 17; i++)
		cpct_setPALColour(i, HW_BLACK);
}

/*****************************************************/
/*													 */
/*	Set display										 */
/*												     */
/*****************************************************/
void SetDisplay(u8 vMem)
{
	sVMem = vMem;
	cpct_setVideoMemoryPage(sVMem == VIDEO_MEM ? cpct_pageC0 : cpct_page80);
}

/*****************************************************/
/*													 */
/*	Get VMem index								 	*/
/*												     */
/*****************************************************/
u8 GetVMemIndex()
{
	return (sVMem == VIDEO_MEM) ? 1 : 0;
}

/*****************************************************/
/*													 */
/*	Flip double buffer								 */
/*												     */
/*****************************************************/
void FlipBuffer()
{
	cpct_waitVSYNC();

	if (sVMem == BUFFER_MEM)
	{
		cpct_setVideoMemoryPage(cpct_pageC0);
		sVMem = VIDEO_MEM;
	}
	else
	{
		cpct_setVideoMemoryPage(cpct_page80);
		sVMem = BUFFER_MEM;
	}
}

/*****************************************************/
/*													 */
/*	Get buffer video pointer						 */
/*												     */
/*****************************************************/
u8* GetBackScreenPtr(u8 xPos, u8 yPos)
{
	return cpct_getScreenPtr(sVMem == VIDEO_MEM ? SCREEN_BUFF : CPCT_VMEM_START, xPos, yPos);
}

/*****************************************************/
/*													 */
/*	Get origin buffer video pointer 				 */
/*												     */
/*****************************************************/
u8* GetOriginBackScreenPtr()
{
	return cpct_getScreenPtr(sVMem == VIDEO_MEM ? SCREEN_BUFF : CPCT_VMEM_START, 0, 0);
}

/*****************************************************/
/*													 */
/*	Get screen video pointer						 */
/*												     */
/*****************************************************/
u8* GetScreenPtr(u8 xPos, u8 yPos)
{
	return cpct_getScreenPtr(sVMem == VIDEO_MEM ? CPCT_VMEM_START : SCREEN_BUFF, xPos, yPos);
}

/*****************************************************/
/*													 */
/*	Get origin screen video pointer					 */
/*												     */
/*****************************************************/
u8* GetOriginScreenPtr()
{
	return GetScreenPtr(0, 0);
}

/*****************************************************/
/*													 */
/*	Clear zone on both screen memory				 */
/*												     */
/*****************************************************/
void ClearScreenZone(i8 posX, u8 posY, u8 cx, u8 cy, u8 buffer)
{
	if ((u8)(posX - SCREEN_CX_BYTE) > (u8)(posX + cx - 1))
	{
		if (posX < 0)
		{
			cx += posX;
			posX = 0;
		}
		else if (posX + cx > SCREEN_CX_BYTE)
		{
			cx = SCREEN_CX_BYTE - posX;
		}

		if (cx > 0 && cy > 0)
		{
			FillScreen(buffer, posX, posY, cx, cy, gRoadColor);
		}
	}
}

/*****************************************************/
/*													 */
/*	Draw sprite										 */
/*												     */
/*****************************************************/
void DrawSpriteType(void* sprite, void* memory, u8 width, u8 height, u8 masked)
{
	if (masked == SPRITE_MASKED)
		cpct_drawSpriteMaskedAlignedTable(sprite, memory, width, height, gMaskTable);
	else
		cpct_drawSprite(sprite, memory, width, height);
}

/*****************************************************/
/*													 */
/*	Draw clipped Sprite								 */
/*												     */
/*****************************************************/
void DrawClippedSprite(const u8* sprite, i8 xPos, u8 yPos, u8 width, u8 height, u8 spriteType, u8 isBackBuffer)
{
	const u8* video;
	TGetBuffer GetBuffer = (isBackBuffer) ? GetBackScreenPtr : GetScreenPtr;

	i8 rightPosX = xPos + width;

	/** No clip */
	if (xPos >= 0 && rightPosX <= SCREEN_CX_BYTE)
	{
		video = GetBuffer(xPos, yPos);
		DrawSpriteType(sprite, video, width, height, spriteType);
	}
	else
	{
		/** Right clip */
		if ((u8)(xPos - width) <= (u8)(xPos - SCREEN_CX_BYTE))
		{
			u8 cx = SCREEN_CX_BYTE - xPos;
			u8 h;
			for (h = 0; h < height; h++)
			{
				const u8* spriteOffset = sprite + (h * width);
				video = GetBuffer(xPos, yPos + h);


				DrawSpriteType(spriteOffset, video, cx, 1, spriteType);
			}
		}
		else
			/** Left clip */
			if (rightPosX > 0 && xPos < 0)
			{
				u8 cx = (u8)rightPosX;
				u8 h;
				for (h = 0; h < height; h++)
				{
					const u8* spriteOffset = sprite - xPos + (h * width);
					video = GetBuffer(0, yPos + h);

					DrawSpriteType(spriteOffset, video, cx, 1, spriteType);
				}
			}
	}
}

/*****************************************************/
/*													 */
/*	Draw clipped RoadObject							 */
/*												     */
/*****************************************************/
void DrawClippedRoadObjectOpaque(SRoadObject* pObj)
{
	const SSpriteInfo* spriteInfo = &pObj->_spriteInfo;
	DrawClippedSprite(spriteInfo->_sprite, spriteInfo->_x, spriteInfo->_y, spriteInfo->_cx, spriteInfo->_cy, SPRITE_OPAQUE, TRUE);
}

/*****************************************************/
/*													 */
/*	Draw sprite on both screen buffers				 */
/*												     */
/*****************************************************/
void DrawSpriteBothBuffers(const u8* pSprite, u8 x, u8 y, u8 cx, u8 cy, u8 masked)
{
	u8 i;
	for (i = 0; i < NB_BUFFER_VIDEO; i++)
	{
		const u8* video = (i == 0) ? GetScreenPtr(x, y) : GetBackScreenPtr(x, y);
		DrawSpriteType(pSprite, video, cx, cy, masked);
	}
}

/*****************************************************/
/*													 */
/*	Get right X position sprite						 */
/*												     */
/*****************************************************/
i8 GetPosRightX(SSpriteInfo* sprite)
{
	return sprite->_x + sprite->_cx;
}

/*****************************************************/
/*													 */
/*	Get right Y position sprite						 */
/*												     */
/*****************************************************/
u8 GetPosRightY(SSpriteInfo* sprite)
{
	return sprite->_y + sprite->_cy;
}

/*****************************************************/
/*													 */
/*	Store sprite current position					 */
/*												     */
/*****************************************************/
void StoreCurrentPos(SSpriteInfo* sprite)
{
	SPos* prevPos = &sprite->_prevPos[GetVMemIndex()];

	if (sprite->_buffIndex < NB_BUFFER_VIDEO)
		sprite->_buffIndex++;

	prevPos->_x = sprite->_x;
	prevPos->_y = sprite->_y;
}